#include "mbed.h"

PwmOut gate_u(PB_3);
PwmOut gate_v(PB_4);
PwmOut gate_w(PB_5);

//int hallpin_1 = PB_6;
//int hallpin_2 = PC_7;
#define hallpin_2 PB_6
#define hallpin_1 PC_7
#define hallpin_3 PC_9

DigitalIn h1(hallpin_1);
DigitalIn h2(hallpin_2);
DigitalIn h3(hallpin_3);

int hall_1_state = h1;
int hall_2_state = h2;
int hall_3_state = h3;

InterruptIn hall_1(hallpin_1);
InterruptIn hall_2(hallpin_2);
InterruptIn hall_3(hallpin_3);

InterruptIn button1(PA_0);

AnalogIn reverse(PA_4);
AnalogIn throttle(PA_6);

// Array for log
//pc.printf("halls %d %d %d quadrant %d theta %f\n", hall_1_state, hall_2_state, hall_3_state, quadrant, theta);
const int NUM_LOGS = 1;
int log_cnt = 0;
float log_detheta_array[NUM_LOGS];
float log_theta_array[NUM_LOGS];

// PWM properties
float pwm_freq = 8000.0f; //hz
float ff;
float pi = 3.14159;
int bin_theta = 0;
int quadrant = 0;
float theta = 0.0f;

float Ia = 0.0f;
float Ib = 0.0f;
float Ic = 0.0f;

float Id = 0.0f;
float Iq = -0.2f;

// setup pwm interrupt. put freq in this area soon. figure out how to do center aligned pwm!!
Ticker pwm_interrupt;
float datheta = 0.0f;

// This board is whack, needs a custom serial setup
Serial pc(PA_2, PA_3);

// hall interrupts
void hall_1_rise(){ hall_1_state = 1; }
void hall_2_rise(){ hall_2_state = 1; }
void hall_3_rise(){ hall_3_state = 1; }

void hall_1_fall(){ hall_1_state = 0; }
void hall_2_fall(){ hall_2_state = 0; }
void hall_3_fall(){ hall_3_state = 0; }

void button_1_rise(){ //pc.printf("detheta %f theta: %f halls: %d %d %d\n", log_detheta_array[0], log_theta_array[0], hall_1_state, hall_2_state, hall_3_state); 
pc.printf("detheta %f theta: %f\n", log_detheta_array[0], log_theta_array[0]);}

void check_commutation() {
    
    bin_theta = hall_1_state<<2 | hall_2_state<< 1 | hall_3_state;
    
    if (bin_theta == 0b001) { quadrant = 5; }
    if (bin_theta == 0b101) { quadrant = 4; }
    if (bin_theta == 0b100) { quadrant = 3; }
    if (bin_theta == 0b110) { quadrant = 2; }
    if (bin_theta == 0b010) { quadrant = 1; }
    if (bin_theta == 0b011) { quadrant = 0; }
    // Position sensor is offset in this case by 180 degrees
    theta = (float)quadrant+3.0f*pi/3.0f;
    
    log_detheta_array[0] = datheta;
    log_theta_array[0] = theta;
    
    Iq = throttle.read() - reverse.read();
    
    //theta = datheta;
    //theta = theta + 6.0f*pi/3.0f;
    Ia = Id*cosf(theta) - Iq*sinf(theta);
    Ib = Id*cosf(theta-(2.0f/3.0f)*pi) - Iq*sinf(theta-(2.0f/3.0f)*pi);
    Ic = Id*cosf(theta+(2.0f/3.0f)*pi) - Iq*sinf(theta+(2.0f/3.0f)*pi);
    
    // This will only give 80% of total available power, eventually use svm
    // https://en.wikipedia.org/wiki/Space_vector_modulation
    gate_u.write(0.5f+Ia/2.0f);
    gate_v.write(0.5f+Ib/2.0f);
    gate_w.write(0.5f+Ic/2.0f);
    
    // log it all
    //pc.printf("halls %d %d %d quadrant %d theta %f\n", hall_1_state, hall_2_state, hall_3_state, quadrant, theta);

    
}

int main() {
    // specify period first, then everything else
    gate_u.period(1.0f/pwm_freq);  // 4 second period
    gate_v.period(1.0f/pwm_freq);  // 4 second period
    gate_w.period(1.0f/pwm_freq);  // 4 second period
    
    gate_u.write(0.0f);  // 50% duty cycle
    gate_v.write(0.0f);  // 50% duty cycle
    gate_w.write(0.0f);  // 50% duty cycle
    
    hall_1.rise(&hall_1_rise);
    hall_1.fall(&hall_1_fall);

    hall_2.rise(&hall_2_rise);
    hall_2.fall(&hall_2_fall);

    hall_3.rise(&hall_3_rise);
    hall_3.fall(&hall_3_fall);
    
    button1.rise(&button_1_rise);
    
    hall_1.mode(PullUp);
    hall_2.mode(PullUp);
    hall_3.mode(PullUp);
    
    // Make this center aligned eventually
    pwm_interrupt.attach(&check_commutation, 1.0f/pwm_freq);
 
    int loop_cnt = 0;
    while (1) {
        //pc.printf("nice\n");
        //pc.printf("Halls: %d %d %d\n", hall_1_state, hall_2_state, hall_3_state);
        //pc.printf("Angle: %f Ia Ib Ic %f %f %f\n", theta, Ia, Ib, Ic);
        //pc.printf("Angle: %f \n", theta);
        //pc.printf("halls %d %d %d quadrant %d theta %f\n", hall_1_state, hall_2_state, hall_3_state, quadrant, theta);
        datheta += 0.05f;
        wait(0.001);
        if (datheta > 2.0f*pi) {
            datheta = 0.0f;
            }
        loop_cnt++;
        if (loop_cnt > 800) {
            //pc.printf("detheta %f theta: %f\n", log_detheta_array[0], log_theta_array[0]);
            loop_cnt = 0;
        }
        //pc.printf("Theta: %f Halls: %d %d %d\n", theta, hall_1_state, hall_2_state, hall_3_state);
        //update_commutation();
        

        //pc.printf("Ia %f Ib %f Ic %f\n", Ia, Ib, Ic);
        //pc.printf("Reverse: %f Throttle: %f\n", reverse.read(), throttle.read());
    }
}
